@@ -182,9 +182,36 @@ expandBasisFunction <- function(scores, argvals = functions@argvals, functions)
 
   # collapse higher-dimensional functions, multiply with scores and resize the result
   d <- dim(functions@X)
-  dim(functions@X) <- c(d[1], prod(d[-1]))
-  resX <- scores %*% functions@X
-  dim(resX) <- c(dim(scores)[1],d[-1])
+  nd <- length(d)
+  
+  if(nd == 2)
+    resX <- scores %*% functions@X
+  
+  if(nd == 3)
+  {
+    resX <- array(NA, dim = c(dim(scores)[1], d[-1]))
+    
+    for(i in 1:d[2])
+      resX[,i,] <- scores %*% functions@X[,i,]
+  }
+  
+  if(nd == 4)
+  {
+    resX <- array(NA, dim = c(dim(scores)[1], d[-1]))
+    
+    for(i in 1:d[2])
+      for(j in 1:d[3])
+          resX[,i,j,] <- scores %*% functions@X[,i,j,]
+  }
+    
+  if(nd > 4) # slow solution due to aperm
+  {
+    resX <- aperm(plyr::aaply(.data = functions@X, .margins = 3:nd, 
+                        .fun = function(x,y){y %*% x}, y = scores), 
+                  c(nd-1,nd, 1:(nd-2)))
+    dimnames(resX) <- NULL
+  }  
+  
   return( funData(argvals, resX) )
 }
 
@@ -460,7 +487,7 @@ dctFunction2D <- function(scores, argvals, parallel = FALSE)
 #'
 #' @seealso \code{\link{dctBasis2D}}
 #'
-#' @useDynLib MFPCA calcImage
+#' @useDynLib MFPCA, .registration = TRUE
 #'
 #' @keywords internal
 idct2D <- function(scores, ind, dim)
@@ -468,6 +495,12 @@ idct2D <- function(scores, ind, dim)
   if(length(dim) != 2)
     stop("Function idct2D can handle only 2D images.")
   
+  if(length(ind) != length(scores))
+    stop("Indices do not match number of scores.")
+  
+  if(length(ind) == 0) # no scores at all
+    return(array(0, dim))
+  
   if(min(ind) < 1)
     stop("Indices must be positive.")
   
@@ -532,7 +565,7 @@ dctFunction3D <- function(scores, argvals, parallel = FALSE)
 #'
 #' @seealso \code{\link{dctBasis3D}}
 #'
-#' @useDynLib MFPCA calcImage3D
+#' @useDynLib MFPCA, .registration = TRUE
 #'
 #' @keywords internal
 idct3D <- function(scores, ind, dim)
@@ -540,6 +573,12 @@ idct3D <- function(scores, ind, dim)
   if(length(dim) != 3)
     stop("Function idct3D can handle only 3D images.")
   
+  if(length(ind) != length(scores))
+    stop("Indices do not match number of scores.")
+  
+  if(length(ind) == 0) # no scores at all
+    return(array(0, dim))
+  
   if(min(ind) < 1)
     stop("Indices must be positive.")
   
